//FFT class library
//P5-150 1024point fft 50msec
#include  "fft.h"
#include <math.h>
/*--------------------------------------------------*/
//#define       DEBUG
/*--------------------------------------------------*/
FFT::FFT(int Bit)
{
  FFT_N = pow(2.0, FFT_BIT=Bit);
  src = new Vec2<double>[FFT_N];
  sin_d = new double[FFT_N/2];
  cos_d = new double[FFT_N/2];
  Mask_d = new double[FFT_N];
  init();
}
/*--------------------------------------------------*/
FFT::~FFT()
{
  delete  []  src;
  delete  []  sin_d;
  delete  []  cos_d;
  delete  []  Mask_d;
}
/*--------------------------------------------------*/
void  FFT::init(void)
{
  int n;
        int     m =     FFT_N/2;
  for(n=0;  n<m;  n++)  {
    sin_d[n] = sin(2.0*PI*n/FFT_N);
    cos_d[n] = cos(2.0*PI*n/FFT_N);
  }

  Mask = MDoNothing;
}
/*--------------------------------------------------*/
void  FFT::SetIntData(int       *p)
{
        for(int n=0;    n<FFT_N;        n++)    {
        src[n].x = *p++;        src[n].y = 0.0;
  }
}
/*--------------------------------------------------*/
void  FFT::GetIntData(int       *p)
{
        for(int n=0;    n<FFT_N;        n++)    *p++ = (int)abs(src[n]);
}
/*--------------------------------------------------*/
//Zero=0dB ġ Dot10=10dB Ʈ
void  FFT::GetDBCnv(int *p, int Dot10, int Zero)  //dB ȯϿ ´.
{
  double  d, zero;
  zero = 1.0/Zero;
        for(int n=0;    n<FFT_N;        n++, p++)  {
    d=power(src[n]);
        *p = (int)((d<=0.0001)? -40 : 10.0*log10(d*zero))*Dot10*0.1;
  }
}
/*--------------------------------------------------*/
void  FFT::Kikaku(void)
{
  double  DC, AC;
  switch(Mask)  {
  case  MDoNothing: DC = FFT_N; AC = FFT_N*0.5; break;
  case  MTriangle:  DC = FFT_N *0.5; AC = FFT_N*16.00/64.0; break;
  case  MSine:      DC = FFT_N *40.73/64.0; AC = FFT_N*20.41/64.0; break;
  case  MHunning:   DC = FFT_N *34.56/64.0; AC = FFT_N*17.28/64.0; break;
  default :   DC = FFT_N *32.00/64.0; AC = FFT_N* 16.00/64.0; break;
  }
  DC = 0.35/DC;  AC = 0.35/AC;
  src[0] *= DC;
        for(int n=1;    n<FFT_N;        n++)    src[n] *= AC;
}
/*--------------------------------------------------*/
void  FFT::bit_rev(void)
{
  int i,  j,  k;
  double swp;
/* bit reverse */
  j= FFT_N / 2;

  for(i=1 ; i<FFT_N ; i++) {
    k= FFT_N;
    if(j  < i)  {
      swp= src[i].x; src[i].x = src[j].x; src[j].x= swp;
      swp= src[i].y; src[i].y = src[j].y; src[j].y= swp;
    }
    for(k= k/2 ;(j>=k) && (k != 0); j= j-k, k=k/2);
    if(k==0) k=1;
    j= j + k;
  }
}
/*--------------------------------------------------*/
void  FFT::fft_cnv(void)
{
  int i,  j,  k,  p,  l,  h,  g,  q;
  double        a, b, sinp, cosp;

  l= FFT_N;  h= 1;

  for(g= 1 ; g <=  FFT_BIT ; g++)  {
    l= l/2;  k= 0;
    for(q= 1;  q <=  h ; q++)  {
      p= 0;
      for(i= k ; i <=  l+k-1 ; i++)  {
        j= i + l;
        a= src[i].x  - src[j].x;
        b= src[i].y  - src[j].y;
        src[i].x = src[i].x + src[j].x;
        src[i].y = src[i].y + src[j].y;

        if(p== 0)  {
          src[j].x = a;  src[j].y= b;
        } else  {
          cosp= cosf(p);  sinp=sinf(p);
          src[j].x = a*cosp + b*sinp;
          src[j].y = b*cosp - a*sinp;
        }
        p= p + h;
      }
      k= k + l + l;
    }
    h= h + h;
  }
  bit_rev();
}
//----------------------------------------------------
// ũ Ǵ
void    FFT::MaskExe(void)
{
        if(Mask != MDoNothing)
        for(int i=0;    i<FFT_N;        i++)    src[i] *=       Mask_d[i];
}
//----------------------------------------------------
void    FFT::DoNothing(void)
{
        Mask = MDoNothing;
}
//----------------------------------------------------
void    FFT::Triangle(void)
{
        int     i,      j;

        if(Mask != MTriangle)   {
                Mask = MTriangle;
          for(i=0, j = FFT_N/2; i < FFT_N/2; i++)
        Mask_d[i+j] = 1.0-(Mask_d[i]    =       (double)i/j);
  }
}
//----------------------------------------------------
void    FFT::Sine(void)
{
        if(Mask != MSine)       {
        Mask = MSine;
          for(int i=0; i < FFT_N; i++)  Mask_d[i]       = 
      sin(PI*(double)i/FFT_N);
  }
}
//----------------------------------------------------
void    FFT::Hunning(void)
{
        if(Mask != MHunning)    {
        Mask = MHunning;
          for(int i=0; i < FFT_N; i++)
        Mask_d[i] = 0.54-0.46*cos(2.0*PI*(double)i/FFT_N);
  }
}
//----------------------------------------------------
void    FFT::Humming(void)
{
        if(Mask != MHumming)    {
        Mask = MHumming;
          for(int i=0; i < FFT_N; i++)
        Mask_d[i]       =       0.5-0.5*cos(2.0*PI*(double)i/FFT_N);
  }
}
//----------------------------------------------------
#ifdef DEBUG
#include <stdio.h>
#include <stdlib.h>
class FFT fft(6);  //64
/*--------------------------------------------------*/
void  gendata(void)
{
  int n;
  double  x;

  for(n=0; n<fft.GetFFT_N(); n++) {  fft.src[n].x = fft.src[n].y = 0.0; }

    //sin 6
  for(n=0; n<fft.GetFFT_N(); n++) {
    x=sin((2.0*PI*6.0*(double)n)/(double)fft.GetFFT_N());
    fft.src[n].x += x;
  }
     //DC 1v
  for(n=0; n<fft.GetFFT_N(); n++) {
    x=1.0;
    fft.src[n].x += x;
  }

}
/*--------------------------------------------------*/
void  main()
{
  int    n, rand();

  gendata();

  fft.DoNothing();
//  fft.Triangle();
//  fft.Sine();
//  fft.Hunning();
//  fft.Humming();

  fft.MaskExe();
  for(n=0; n<fft.GetFFT_N(); n++) {
//    printf("%f,%f\n", fft.src[n].x, fft.src[n].y);
    printf("%f,", fft.src[n].x);
  }
  puts("------------");
  fft.fft_cnv();
  fft.Kikaku();
  //    for(n=0; n<1000; n++)     fft.fft_cnv();

  for(n=0; n<fft.GetFFT_N(); n++) {
    printf("%f ", abs(fft.src[n]));
  }
}
/*--------------------------------------------------*/
#endif  //DEBUG

